/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcoutputdialog.h"

#include "../kdcfilefactory.h"
#include "../kdcconfig.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QTime>
#include <QWidget>
#include <QHeaderView>
#include <QFrame>
#include <QPalette>
#include <QVariant>

#include <KIcon>
#include <KLocale>
#include <KMenu>
#include <KActionMenu>
#include <KMessageBox>
#include <KPushButton>
#include <KColorScheme>
#include <KDialog>
#include <kio/job.h>

KDCOutputDialog::KDCOutputDialog( QWidget *parent )
	: KDialog( parent,  Qt::WindowTitleHint | Qt::WindowSystemMenuHint )
{
	setWindowIcon( KDCFileFactory::appIcon() );
	p_currentSize = 0;

	p_info = new QTreeWidget;
	p_info->setColumnCount( 3 );
	p_info->header()->hide();
	p_info->setColumnWidth( 0, 30 );
	p_info->setColumnWidth( 1, 100 );
	p_info->setRootIsDecorated( FALSE );

	p_jobProgress = new KDCProgressBar;
	p_jobProgress->setValue( 0 );
	connect( p_jobProgress, SIGNAL( percentChanged( int, int ) ), this, SLOT( on_jobPercentChanged( int, int ) ) );
	p_jobsProgress = new KDCProgressBar;
	p_jobsProgress->setValue( 0 );
	p_cpuUsage = new QProgressBar;
	p_cpuUsage->setRange( 0, 1000 );
	p_speed = new KDCLabel( i18n("No Info") );

	QPalette p;
	p.setColor( QPalette::Window, p.color( QPalette::Highlight ) );

	p_timeInfoStr = "<b>" + i18n("Elapsed time:") + "</b> %1 / <b>" + i18n("Remaining time:") + "</b> %2";
	p_timeInfo = new KDCLabel( p_timeInfoStr );
	p_timeInfo->setAutoFillBackground( TRUE );
	p_timeInfo->setFrameShadow( QFrame::Sunken );
	p_timeInfo->setFrameShape( QFrame::StyledPanel );
	p_timeInfo->setLineWidth( 25 );
	p_timeInfo->setPalette( p );
	p_timeInfo->setColor( Qt::white );
	p_timeInfo->setText( p_timeInfoStr.arg( "00:00:00", "00:00:00" ) );

	QVBoxLayout *vLayout_1 = new QVBoxLayout;
	vLayout_1->addWidget( new KDCLabel( i18n("Output Informations:"), KDCLabel::TITLE ) );
	vLayout_1->addWidget( p_info );
	vLayout_1->addSpacing( 5 );
	vLayout_1->addWidget( p_timeInfo );
	vLayout_1->addSpacing( 5 );
	vLayout_1->addWidget( new KDCLabel( i18n("Process Progress:"), KDCLabel::TITLE ) );
	vLayout_1->addWidget( p_jobProgress );
	vLayout_1->addWidget( new KDCLabel( i18n("Overall Progress:"), KDCLabel::TITLE ) );
	vLayout_1->addWidget( p_jobsProgress );

	QFrame *frame = new QFrame;
	frame->setLineWidth( 45 );
	frame->setFrameShape( QFrame::StyledPanel );
	frame->setAutoFillBackground( TRUE );
	frame->setFrameShadow( QFrame::Sunken );
	frame->setPalette( p );

	p_outputDirWidget = new KDCOutputDirStatusBar;
	p_outputDirWidget->setPalette( p );
	p_outputDirWidget->setIconSize( 22 );
	p_outputDirWidget->setTextColor( Qt::white );
	p_outputDirWidget->start();
	p_outputPath = new KDCLabel( "<b>" + i18n("Output path: ") + "</b> %1", Qt::white );
	p_outputPath->setPointSize( 10 );

	QVBoxLayout *vLayout_2 = new QVBoxLayout( frame );
	vLayout_2->addStretch();
	vLayout_2->addWidget( p_outputPath );
	vLayout_2->addWidget( p_outputDirWidget );
	vLayout_2->addStretch();

	vLayout_1->addSpacing( 5 );
	vLayout_1->addWidget( frame );

	QGridLayout *gLayout = new QGridLayout;
	gLayout->addWidget( new KDCLabel( i18n("Speed:"), KDCLabel::TITLE ), 0, 0 );
	gLayout->addWidget( p_speed, 0, 1 );
	gLayout->addWidget( new KDCLabel( "fps" ), 0, 2 );
	gLayout->setHorizontalSpacing( 20 );
	QFrame *frame_1 = new QFrame;
	frame_1->setFrameShadow( QFrame::Sunken );
	frame_1->setLineWidth( 2 );
	frame_1->setFrameShape( QFrame::VLine );
	gLayout->addWidget( frame_1, 0, 3 );
	gLayout->addWidget( new KDCLabel( i18n("Cpu(s) usage:"), KDCLabel::TITLE ), 0, 4 );
	gLayout->addWidget( p_cpuUsage, 0, 5 );

	gLayout->setColumnStretch( 0, 1 );
	gLayout->setColumnStretch( 1, 2 );
	gLayout->setColumnStretch( 3, 1 );
	gLayout->setColumnStretch( 4, 1 );
	gLayout->setColumnStretch( 5, 10 );

	vLayout_1->addSpacing( 5 );
	vLayout_1->addLayout( gLayout );
	vLayout_1->addStretch();

	QWidget *mainWidget = new QWidget;
	mainWidget->setLayout( vLayout_1 );
	setMainWidget( mainWidget );

	KActionMenu *p_options = new KActionMenu( this );
	p_previewAction = new KAction( KIcon("media-playback-start"), i18n("&Preview"), this );
	p_previewAction->setCheckable( TRUE );
	p_previewAction->setData( QVariant( FALSE ) );
	connect( p_previewAction, SIGNAL( triggered( bool ) ), this, SLOT( setPreviewChecked( bool ) ) );
	p_turnOffAction = new KAction( KIcon("system-log-out"), i18n("Shut &down the computer after all work has been done"), this );
	p_turnOffAction->setCheckable( TRUE );
	p_turnOffAction->setEnabled( FALSE );
	p_options->addAction( p_previewAction );
	p_options->addAction( p_turnOffAction );

	setButtons( KDialog::Close | KDialog::User1 | KDialog::User2 );
	showButtonSeparator( TRUE );
	enableButton( KDialog::Close, FALSE );
	connect( this, SIGNAL( closeClicked() ), this, SLOT( close() ) );

	setButtonText( KDialog::User2, i18n("&Options") );
	setButtonIcon( KDialog::User2, KIcon("configure") );
	button( KDialog::User2 )->setMenu( dynamic_cast<QMenu*>( p_options->menu() ) );

	setButtonText( KDialog::User1, i18n("&Stop") );
	setButtonIcon( KDialog::User1, KIcon("process-stop") );
	connect( this, SIGNAL( user1Clicked() ), this, SLOT( stop() ) );

	p_elapsedTime = new QTime;
	p_elapsedTime->start();
	p_timer = new QBasicTimer;
	p_timer->start( KDCConfig::updateGui(), this );

	setMinimumSize( 650, 500 );
	setMaximumSize( 650, 500 );
}

KDCOutputDialog::~KDCOutputDialog()
{
	p_timer->stop();
	delete p_timer;
	delete p_elapsedTime;
	qDebug( "~KDCOutputDialog" );
}

void KDCOutputDialog::timerEvent( QTimerEvent *event )
{
	if ( event->timerId() == p_timer->timerId() )
	{
		p_jobProgress->refresh();
		QTime time;
		int speed = p_jobSpeed;
		if ( speed == 0 )
			speed = p_currentSize - p_previousSize;
		QString str = KIO::calculateRemaining( p_totalSize, p_currentSize, speed ).toString( "hh:mm:ss" );
		p_timeInfo->setText( p_timeInfoStr.arg( time.addMSecs( p_elapsedTime->elapsed() ).toString( "hh:mm:ss" ), str ) );

		p_speed->setText( QString::number( speed ) );
		p_jobsProgress->refresh();
		QString title = QString( "(%1%) - %2 - KDVDCreator" ).arg( QString::number( p_jobsProgress->percent() ), str );
		setWindowTitle( title );
		p_previousSize = p_currentSize;
		p_cpuUsage->setValue( p_cpu );
	}
	else
		QWidget::timerEvent( event );
}

void KDCOutputDialog::setJobProgress( int val )
{
	p_currentSize += val - p_jobProgress->currentValue();
	p_jobProgress->setNewValue( val );
}

void KDCOutputDialog::setJobPercent( int val )
{
	p_jobProgress->setPercent( val );
}

void KDCOutputDialog::setJobTotalSize( int val )
{
	qDebug( "KDCOutputDialog::setJobTotalSize( %d )", val );
	p_jobProgress->setTotalSize( val );
}

void KDCOutputDialog::setTotalSize( int val )
{
	p_totalSize = val;
}

void KDCOutputDialog::setJobCount( int val )
{
	qDebug( "KDCOutputDialog::setJobCount( %d )", val );
	p_jobsProgress->setTotalSize( val*100 );
}

void KDCOutputDialog::setJobSpeed( double val )
{
	p_jobSpeed = val;
}

void KDCOutputDialog::setDescription( const QString& description, KDCOutputDialog::Description mode )
{
	qDebug( description.toUtf8() );
	QTreeWidgetItem *item = new QTreeWidgetItem;
	item->setFlags( Qt::ItemIsEnabled );
	item->setTextAlignment( 0, Qt::AlignHCenter | Qt::AlignVCenter );
	item->setTextAlignment( 1, Qt::AlignHCenter | Qt::AlignVCenter );
	item->setText( 2, description );
	p_info->addTopLevelItem( item );
	if ( mode == KDCOutputDialog::Information )
		item->setIcon( 0, KIcon("dialog-information") );
	else
		if ( mode == KDCOutputDialog::Abort )
			item->setIcon( 0, KIcon("process-stop") );
	else
		if ( mode == KDCOutputDialog::Error )
			item->setIcon( 0, KIcon("dialog-error") );
	else
		item->setIcon( 0, KIcon("dialog-ok") );
	item->setText( 1, QTime::currentTime().toString( "hh:mm:ss" ) );
	p_info->setCurrentItem( item );
}

void KDCOutputDialog::stop()
{
	int answer = KMessageBox::questionYesNo( this, i18n( "Do you really want to abort currently process?") );
	if ( answer == KMessageBox::Yes )
	{
		abortProcess();
		emit abort();
	}
}

void KDCOutputDialog::abortProcess()
{
	p_timer->stop();
	enableButton( KDialog::User2, FALSE );
	enableButton( KDialog::User1, FALSE );
	enableButton( KDialog::Close, TRUE );
	setDescription( i18n("Process was aborded"), KDCOutputDialog::Abort );
	p_outputDirWidget->stop();
	setPreviewChecked( FALSE );
}

void KDCOutputDialog::jobsDone()
{
	p_timer->stop();
	p_outputDirWidget->stop();
	p_jobsProgress->setValue( p_jobsProgress->maximum() );
	p_jobProgress->setValue( p_jobsProgress->maximum() );
	setWindowTitle( "(100) - 00:00:00 - KDVDCreator" );
	QTime time;
	p_timeInfo->setText( p_timeInfoStr.arg( time.addMSecs( p_elapsedTime->elapsed() ).toString( "hh:mm:ss" ), "00:00:00" ) );
	setDescription( i18n("All jobs sucesfully finished"), KDCOutputDialog::Done );

	enableButton( KDialog::User2, FALSE );
	enableButton( KDialog::User1, FALSE );
	enableButton( KDialog::Close, TRUE );
}

void KDCOutputDialog::setDiscInfo( const QString& path, quint64 size )
{
	p_outputPath->setText( p_outputPath->text().arg( path ) );
	p_outputDirWidget->setProjectInfo( path, size );
}

void KDCOutputDialog::on_jobPercentChanged( int oldPercent, int newPercent )
{
	p_jobsProgress->setProgress( p_jobsProgress->currentValue() + newPercent - oldPercent );
}

void KDCOutputDialog::closeEvent( QCloseEvent *event )
{
	if ( isButtonEnabled( KDialog::Close ) )
	{
		emit closePreview();
		event->accept();
		deleteLater();
	}
	else
		event->ignore();
}

void KDCOutputDialog::setPreviewChecked( bool enabled )
{
	qDebug( "KDCOutputDialog::setPreviewEnabled( %d )::start::prev = %d ", QVariant( enabled ).toInt(), p_previewAction->data().toInt() );
	p_previewAction->setChecked( enabled );
	if ( enabled && !p_previewAction->data().toBool() )
	{
		p_previewAction->setData( QVariant( TRUE ) );
		qDebug( "KDCOutputDialog::setPreviewEnabled()::emit showPreview()" );
		emit showPreview();
	}
	if ( !enabled && p_previewAction->data().toBool() )
	{
		p_previewAction->setData( QVariant( FALSE ) );
		qDebug( "KDCOutputDialog::setPreviewEnabled()::emit closePreview()" );
		emit closePreview();
	}
	qDebug( "KDCOutputDialog::setPreviewEnabled()::end" );
}

void KDCOutputDialog::uncheckPreview()
{
	p_previewAction->setChecked( FALSE );
	p_previewAction->setData( QVariant( FALSE ) );
}

void KDCOutputDialog::setPreviewEnabled( bool enabled )
{
	p_previewAction->setEnabled( enabled );
}

void KDCOutputDialog::setCpuUsage( int val )
{
	p_cpu = val;
}
