/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcbuildallcompositejob.h"

#include "kdcbuilddisccompositejob.h"

#include <QListIterator>
#include <QStringList>

#include <KDiskFreeSpace>
#include <kio/job.h>

KDCBuildAllCompositeJob::KDCBuildAllCompositeJob( const QList<KDCDisc*>& list, QObject *parent )
	: KDCCompositeJob( KDCCompositeJob::No, parent )
{
	KDCBuildDiscCompositeJob *job;
	QListIterator<KDCDisc*> it( list );
	while ( it.hasNext() )
	{
		job = new KDCBuildDiscCompositeJob( it.next() );
		connect( job, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
		connect( job, SIGNAL( speed( double ) ), this, SLOT( on_jobSpeed( double ) ) );
		connect( job, SIGNAL( progress( int ) ), this, SLOT( on_jobProgress( int ) ) );
		connect( job, SIGNAL( percent( int ) ), this, SLOT( on_jobPercent( int ) ) );
		connect( job, SIGNAL( jobTotalSize( int ) ), this, SLOT( on_jobTotalSize( int ) ) );
		connect( job, SIGNAL( infoMessage( const QString& ) ), this, SLOT( on_jobInfoMessage( const QString& ) ) );
		connect( job, SIGNAL( errorMessage( const QString& ) ), this, SLOT( on_jobErrorMessage( const QString& ) ) );
		connect( job, SIGNAL( discInfoMessage( const QString& , quint64 ) ), this, SLOT( on_discInfoMessage( const QString& , quint64 ) ) );
		connect( job, SIGNAL( movieChanged( QTreeWidgetItem * ) ), this, SLOT( on_movieChanged( QTreeWidgetItem * ) ) );
		connect( job, SIGNAL( jobMainProgram( const QString& ) ), this, SLOT( on_jobMainProgram( const QString& ) ) );
		addSubjob( job );
	}

	p_dlg = new KDCOutputDialog;
	connect( p_dlg, SIGNAL( abort() ), this, SLOT( deleteLater() ) );
	p_dlg->setTotalSize( totalSize() );
	p_dlg->setJobCount( count() );
	p_dlg->setModal( TRUE );
	p_dlg->show();

	p_cpuInfo = new KDCCpuInfoJob( this );
	connect( p_cpuInfo, SIGNAL( info( int ) ), p_dlg, SLOT( setCpuUsage( int ) ) );
}

KDCBuildAllCompositeJob::~KDCBuildAllCompositeJob()
{
	if ( p_cpuInfo )
		delete p_cpuInfo;
	emit done();
}

void KDCBuildAllCompositeJob::start()
{
	firstSubjob()->start();
	p_cpuInfo->start();
}

void KDCBuildAllCompositeJob::on_jobFinished( KDCJob *job )
{
	if ( job->hasErrors() )
	{
		removeSubjob( job );
		p_dlg->abortProcess();
		deleteLater();
	}
	else
	{
		removeSubjob( job );
		if ( hasSubjobs() )
			firstSubjob()->start();
		else
		{
			p_dlg->jobsDone();
			deleteLater();
		}
	}
}

KDialog* KDCBuildAllCompositeJob::dialog() const
{
	return p_dlg;
}

void KDCBuildAllCompositeJob::on_jobInfoMessage( const QString& text )
{
	p_dlg->setDescription( text );
}

void KDCBuildAllCompositeJob::on_jobProgress( int val )
{
	p_dlg->setJobProgress( val );
	emit progress( val );
}

void KDCBuildAllCompositeJob::on_discInfoMessage( const QString& path, quint64 size )
{
	p_dlg->setDiscInfo( path, size );
}

void KDCBuildAllCompositeJob::on_jobPercent( int val )
{
	p_dlg->setJobPercent( val );
}

void KDCBuildAllCompositeJob::on_jobSpeed( double val )
{
	p_dlg->setJobSpeed( val );
}

void KDCBuildAllCompositeJob::on_jobTotalSize( int val )
{
	p_dlg->setJobTotalSize( val );
}

void KDCBuildAllCompositeJob::on_jobErrorMessage( const QString& text )
{
	QStringList list = text.split( '\t' );
	for ( int i=0; i<list.size()-1; ++i )
		if ( list.at( i ) != "0" )
			p_dlg->setDescription( list.at( i ), KDCOutputDialog::Error );
}

void KDCBuildAllCompositeJob::on_movieChanged( QTreeWidgetItem *movie )
{
	emit processingMovieChanged( movie );
	if ( !movie )
		p_dlg->setPreviewEnabled( FALSE );
	else
		p_dlg->setPreviewEnabled( TRUE );
}

void KDCBuildAllCompositeJob::on_jobMainProgram( const QString& program )
{
	if ( p_cpuInfo )
		p_cpuInfo->setProgram( program );
}
