/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcdvdauthorcreatexmlfilejob.h"

#include "../kdcfilefactory.h"

#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QTextStream>
#include <QList>
#include <QFile>
#include <QTreeWidgetItem>
#include <QStringList>

#include <KLocale>

KDCDVDAuthorCreateXMLFileJob::KDCDVDAuthorCreateXMLFileJob( const KDCDVDDisc *disc, QObject *parent )
	: KDCJob( parent )
{
	p_file = KDCFileFactory::discConfigFile( disc );
	p_doc = new QDomDocument;
	QDomElement dvdauthor = p_doc->createElement( "dvdauthor" );
	dvdauthor.setAttribute( "dest", KDCFileFactory::discOutputDirectory( disc ) + "/disc" );
	QDomElement vmgm = p_doc->createElement( "vmgm" );
	p_doc->appendChild( dvdauthor );
	dvdauthor.appendChild( vmgm );

	QDomElement titleset;
	QDomElement titles;
	QDomElement pgc;
	QDomElement vob;

	QList<QTreeWidgetItem*> movies;
	QList<QTreeWidgetItem*>::const_iterator it;
	int j;
	KDCMovie *movie;
	KDCMovieGroup *group;
	KDCChapterGroup *chapters;
	QStringList chapterList;
	for ( int i=0; i<disc->vtsCount(); ++i )
	{
		titleset = p_doc->createElement( "titleset" );
		dvdauthor.appendChild( titleset );
		titles = p_doc->createElement( "titles" );
		pgc = p_doc->createElement( "pgc" );
		titleset.appendChild( titles );
		titles.appendChild( pgc );
		movies = disc->vts( i )->movies();
		for( it = movies.constBegin(); it != movies.constEnd(); ++it )
		{
			vob = p_doc->createElement( "vob" );
			if ( ( *it )->type() == KDC::Movie )
			{
				movie = static_cast<KDCMovie*>( *it );
				vob.setAttribute( "file", KDCFileFactory::movieName( movie ) );
				chapters = movie->chapterGroup();
			}
			else
			{
				group = static_cast<KDCMovieGroup*>( *it );
				vob.setAttribute( "file", KDCFileFactory::movieName( group ) );
				chapters = group->chapterGroup();
			}
			if ( chapters->count() > 0 )
			{
				chapterList.clear();
				chapterList << "0";
				for ( j=0; j<chapters->count(); ++j )
					chapterList << chapters->chapterAsString( i );
				vob.setAttribute( "chapters", chapterList.join(",") );
			}
			pgc.appendChild( vob );
		}
	}

	setTotalSize( 0 );
}

KDCDVDAuthorCreateXMLFileJob::~KDCDVDAuthorCreateXMLFileJob()
{
	delete p_doc;
}

void KDCDVDAuthorCreateXMLFileJob::start()
{
	qDebug( "KDCDVDAuthorCreateXMLFileJob::start" );
	emit jobTotalSize( 0 );
	emit infoMessage( i18n("Start creating xml file for DVDAuthor") );
	emit percent( 0 );
	QFile out( p_file );
	if ( !out.open( QIODevice::WriteOnly | QIODevice::Truncate ) )
	{
		setError( i18n("There was an error during process.") );
		emit finished( this );
	}
	else
	{
		QTextStream st( &out );
		st << p_doc->toString();
		out.close();
		emit infoMessage( i18n("Creating xml file for DVDAuthor sucesfully finished") );

		qDebug( "KDCDVDAuthorCreateXMLFileJob::end" );
		emit finished( this );
	}
}
