/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcreadfilecompositejob.h"

#include <QDebug>

#include "../kdcfilefactory.h"
#include "../kdcconfig.h"

#include "kdcgrabframejob.h"
#include "kdcconvertframejob.h"
#include "kdcbuildframejob.h"
#include "kdcdecorateframejob.h"
#include "kdcreadfileinfojob.h"


KDCReadFileCompositeJob::KDCReadFileCompositeJob( const QStringList& files, KDCDisc *disc, QObject *parent )
	: KDCCompositeJob( KDCCompositeJob::No, parent )
{
	qDebug("q");
	p_disc = disc;
	p_totalProgress = 0;
	p_movie = NULL;
	p_data = TRUE;

	KDCReadFileInfoJob *job;
	QStringList::const_iterator it;
	for ( it = files.constBegin(); it != files.constEnd(); ++it )
	{
		qDebug("w");
		job = new KDCReadFileInfoJob( *it, this );
		connect( job, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
		connect( job, SIGNAL( percent( int ) ), this, SLOT( on_jobPercent( int ) ) );
		connect( job, SIGNAL( infoMessage( const QString& ) ), this, SLOT( on_jobInfoMessage( const QString& ) ) );
		connect( job, SIGNAL( description( const QString& ) ), this, SLOT( on_jobDescription( const QString& ) ) );
		connect( job, SIGNAL( movieInfo(	const QString& , KDCVideoStreamMetaData *,
																			const QList<KDCAudioStreamMetaData*>& ) ), this,
									SLOT( on_movieInfo(	const QString& , KDCVideoStreamMetaData *,
																			const QList<KDCAudioStreamMetaData*>& ) ) );
		connect( job, SIGNAL( noVideoStreamFound() ), this, SLOT( on_noVideoStreamFound() ) );
		addSubjob( job );
		qDebug("e");
	}
	qDebug("r");
	p_dlg = new KDCProgressDialog;
	qDebug("t");
	qDebug() << p_dlg->progressBar();
	p_dlg->progressBar()->setMaximum( 5*totalSize() );
	qDebug("a");
}

KDCReadFileCompositeJob::~KDCReadFileCompositeJob()
{
	if ( p_dlg )
		delete p_dlg;
}

void KDCReadFileCompositeJob::start()
{
	qDebug("-1-");
	p_disc->blockSignals( TRUE );
	qDebug("-2-");
	firstSubjob()->start();
	qDebug("-3-");
}

void KDCReadFileCompositeJob::on_noVideoStreamFound()
{
	p_totalProgress += 400;
	p_dlg->progressBar()->setValue( p_totalProgress );
	p_data = FALSE;
}

void KDCReadFileCompositeJob::on_jobFinished( KDCJob *job )
{
	if ( job->hasErrors() )
	{
		p_dlg->setError( job->error() );
		p_dlg->exec();
		delete p_dlg;
		removeSubjob( job );
		if ( p_movie )
			p_disc->deleteMovie( p_movie );
		p_disc->blockSignals( FALSE );
		p_disc->refresh();
		emit done( FALSE );
		deleteLater();
	}
	else
	{
		removeSubjob( job );

		if ( hasSubjobs() )
			firstSubjob()->start();
		else
		{
			if ( p_movie )
				p_disc->setCurrentItem( p_movie );
			p_disc->blockSignals( FALSE );
			p_disc->refresh();
			emit done( p_data );
			deleteLater();
		}
	}
}

void KDCReadFileCompositeJob::on_movieInfo( const QString& fileName, KDCVideoStreamMetaData *video,
																						const QList<KDCAudioStreamMetaData*>& audio )
{
	p_movie = new KDCMovie( fileName, video, audio );
	if ( KDCConfig::add() )
		p_movie->checkSubtitles();
	p_disc->addMovie( p_movie );

	KDCDecorateFrameJob *dJob = new KDCDecorateFrameJob(	KDCFileFactory::discDirectory( p_disc ), QString("tmp1.png"),
																												KDCFileFactory::movieFrame( p_movie ), this );
	connect( dJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
	connect( dJob, SIGNAL( infoMessage( const QString& ) ), this, SLOT( on_jobInfoMessage( const QString& ) ) );
	connect( dJob, SIGNAL( percent( int ) ), this, SLOT( on_jobPercent( int ) ) );
	insertSubjob( dJob, 1 );

	KDCBuildFrameJob *bJob = new KDCBuildFrameJob(	KDCFileFactory::discDirectory( p_disc ), QString("tmp.png"),
																									QString("tmp1.png"), this );
	connect( bJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
	connect( bJob, SIGNAL( infoMessage( const QString& ) ), this, SLOT( on_jobInfoMessage( const QString& ) ) );
	connect( bJob, SIGNAL( percent( int ) ), this, SLOT( on_jobPercent( int ) ) );
	insertSubjob( bJob, 1 );

	KDCConvertFrameJob *cJob = new KDCConvertFrameJob(	KDCFileFactory::discDirectory( p_disc ),
																											QString("00000001.jpg"), QString("tmp.png"),
																											184, qRound( 184.0/video->aspectRatio() ), this );
	connect( cJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
	connect( cJob, SIGNAL( infoMessage( const QString& ) ), this, SLOT( on_jobInfoMessage( const QString& ) ) ); 
	connect( cJob, SIGNAL( percent( int ) ), this, SLOT( on_jobPercent( int ) ) );
	insertSubjob( cJob, 1 );

	KDCGrabFrameJob *gJob = new KDCGrabFrameJob(	qRound( video->length()/3000.0 ), p_movie->fileName(),
																								KDCFileFactory::discDirectory( p_disc ), this );
	connect( gJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
	connect( gJob, SIGNAL( infoMessage( const QString& ) ), this, SLOT( on_jobInfoMessage( const QString& ) ) );
	connect( gJob, SIGNAL( percent( int ) ), this, SLOT( on_jobPercent( int ) ) );
	insertSubjob( gJob, 1 );
}

void KDCReadFileCompositeJob::on_jobPercent( int percent )
{
	p_totalProgress += percent;
	p_dlg->progressBar()->setValue( p_totalProgress );
}

void KDCReadFileCompositeJob::on_jobInfoMessage( const QString& text )
{
	p_dlg->setSubTitle( text );
}

void KDCReadFileCompositeJob::on_jobDescription( const QString& text )
{
	p_dlg->setMainTitle( KDCFileFactory::fileName( text ) );
}
