/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdccvddisc.h"

KDCCVDDisc::KDCCVDDisc( int id, QWidget *parent ) 
	: KDCXVCDDisc( KDC::CVD, id, parent )
{
	p_max_subtitleStreams = 4;
	p_max_audioStreams = 2;
}

void KDCCVDDisc::addMovie( KDCMovie *movie )
{
	addMoviePrivate( movie );

	KDCVideoStreamMetaData *video = movie->videoStream()->metaData();

	int i;
	movie->videoStream()->setCodec( KDC::mpeg2 );
	if ( movie->videoFormat() == KDC::PAL ) //PAL
	{	
		// Video
		if ( KDCVideoStream::isCVDPALVideoStream( video ) )
			movie->videoStream()->setMode( KDC::Copy );
		else
		{
			movie->videoStream()->setMode( KDC::Resample );
			movie->videoStream()->setBitrate( 1500 );
			movie->videoStream()->setResolution( 352, 576 );
			movie->videoStream()->setAspectRatio( KDC::LetterBox );
			movie->setVideoFormat( KDC::PAL );
		}
	}
	else //NTSC
	{
		if ( KDCVideoStream::isSVCDNTSCVideoStream( video ) )
			movie->videoStream()->setMode( KDC::Copy );
		else
		{
			movie->videoStream()->setMode( KDC::Resample );
			movie->videoStream()->setBitrate( 1500 );
			movie->videoStream()->setResolution( 352, 480 );
			movie->setVideoFormat( KDC::NTSC );
			movie->videoStream()->setAspectRatio( KDC::LetterBox );
		}
	}
	// Audio
	KDCAudioStreamGroup *group = movie->audioStreamGroup();
	KDCAudioStreamMetaData *audio;
	if ( group )
	{
		for( i=0; i<group->count(); ++i )
		{
			audio = group->stream( i )->metaData();
			if ( KDCAudioStream::isSVCDAudioStream( audio ) )
				group->stream( i )->setMode( KDC::Copy );
			else
			{
				group->stream( i )->setMode( KDC::Resample );
				group->stream( i )->setCodec( KDC::MP2 );
				group->stream( i )->setChannelCount( 2 );
				group->stream( i )->setBitrate( 224 );
				group->stream( i )->setDelay( 0 );
			}
			group->stream( i )->setEnabledMode( KDC::Enabled );
		}
	}

	connect( movie, SIGNAL( settingsChanged() ), this, SLOT( computeNewSize() ) );

	movie->videoStream()->setUpdatesMode( KDC::Enabled );
	movie->videoStream()->refresh();
	if ( group )
	{
		for( i=0; i<group->count(); ++i )
		{
			group->stream( i )->setUpdatesMode( KDC::Enabled );
			group->stream( i )->refresh();
		}
	}
	movie->setUpdatesMode( KDC::Enabled );
	movie->refresh();
}


QString KDCCVDDisc::name() const
{
	return QString("CVD");
}
