/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcdiscsettings.h"

#include "kdcdisc.h"
#include "kdcconfig.h"

#include <QPalette>

#include <KIcon>
#include <KLocale>
#include <KFileDialog>
#include <KUrl>

KDCDiscSettings::KDCDiscSettings( KDCDisc *disc )
	: QObject( 0 )
{
	p_parent = disc;
	p_outputPath = KDCConfig::defaultOutputPath();
	p_deleteEnabled = FALSE;
	p_structureEnabled = FALSE;
	p_deleteTempEnabled = FALSE;
	disc->setAnimated( KDCConfig::animation() ); 

	p_menu = new KMenu( i18n("Disc options") );
	p_menu->setIcon( KIcon("configure") );
	KAction *action = new KAction( KIcon("document-save"), i18n("Set Disc Output Path"), this );
	p_menu->addAction( action );
	connect( action, SIGNAL( triggered() ), this, SLOT( on_setOutputPath() ) );

	KSelectAction *mediaSelectAction = new KSelectAction( i18n("Select Media Size..."), this );
	p_menu->addAction( mediaSelectAction );

	if ( disc->discType() == KDC::DVD )
	{
		p_mediaSize = KDC::DVD47;
		action = mediaSelectAction->addAction( i18n("DVD 4.7 GB") );
		action->setChecked( TRUE );
		connect( action, SIGNAL( triggered() ), this, SLOT( on_mediaSizeChanged() ) );
		p_mediaActions.insert( KDC::DVD47, action );

		action = mediaSelectAction->addAction( i18n("DVD 8 GB") );
		connect( action, SIGNAL( triggered() ), this, SLOT( on_mediaSizeChanged() ) );
		p_mediaActions.insert( KDC::DVD8, action );
	}
	else
	{
		p_mediaSize = KDC::CD700;
		action = mediaSelectAction->addAction( i18n("CD 700 MB") );
		action->setChecked( TRUE );
		connect( action, SIGNAL( triggered() ), this, SLOT( on_mediaSizeChanged() ) );
		p_mediaActions.insert( KDC::CD700, action );

		action = mediaSelectAction->addAction( i18n("CD 800 MB") );
		connect( action, SIGNAL( triggered() ), this, SLOT( on_mediaSizeChanged() ) );
		p_mediaActions.insert( KDC::CD800, action );
	}
}

KDCDiscSettings::~KDCDiscSettings()
{
	QMap<KDC::MediaSize, KAction*>::iterator it;
	for ( it = p_mediaActions.begin(); it != p_mediaActions.end(); ++it )
		delete it.value();
	delete p_menu;
}

KDC::MediaSize KDCDiscSettings::mediaSize() const
{
	return p_mediaSize;
}

QString KDCDiscSettings::outputPath() const
{
	return p_outputPath;
}

void KDCDiscSettings::setOutputPath( const QString& path )
{
	p_outputPath = path;
	if ( p_outputPath.endsWith( '/' ) )
		p_outputPath.resize( p_outputPath.size() - 1 );
	emit discOutputDirectoryChanged();
}

void KDCDiscSettings::setDeleteFilesEnabled( bool enabled )
{
	p_deleteEnabled = enabled;
}

bool KDCDiscSettings::deleteFilesEnabled()
{
	return p_deleteEnabled;
}

void KDCDiscSettings::setDeleteTempFilesEnabled( bool enabled )
{
	p_deleteTempEnabled = enabled;
}

bool KDCDiscSettings::deleteTempFilesEnabled()
{
	return p_deleteTempEnabled;
}

void KDCDiscSettings::setStructureEnabled( bool enabled )
{
	p_structureEnabled = enabled;
}

bool KDCDiscSettings::structureEnabled()
{
	return p_structureEnabled;
}

KMenu* KDCDiscSettings::menu() const
{
	return p_menu;
}

void KDCDiscSettings::on_mediaSizeChanged()
{
	KAction *action = qobject_cast<KAction*>( sender() );
	if ( action )
	{
		p_mediaSize = p_mediaActions.key( action );
		emit mediaSizeChanged();
	}
}

void KDCDiscSettings::on_setOutputPath()
{
	KUrl path = KFileDialog::getExistingDirectoryUrl( KUrl( p_outputPath ), 0, i18n("Select Output Directory") );
	if ( !path.isEmpty() )
		setOutputPath( path.toLocalFile() );
}
