/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcchaptereditor.h"

#include "../kdcfilefactory.h"
#include "../lib/kdclib.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QRadioButton>
#include <QStringList>
#include <QListWidgetItem>
#include <QPixmap>
#include <QTime>

#include <KIcon>
#include <KLocale>
#include <KMessageBox>
#include <KAction>

KDCChapterEditor::KDCChapterEditor( KDCMPlayerWidget *mediaPlayer, QWidget *parent ) : QWidget( parent )
{
	p_movie = NULL;
	p_group = NULL;
	p_id = 1;
	p_mediaPlayer = mediaPlayer;

	p_viewMode = new QButtonGroup( this );

	QHBoxLayout *mainLayout = new QHBoxLayout( this );
	p_buttonGroup = new KButtonGroup;
	p_buttonGroup->setTitle( i18n("Chapters") );
	p_buttonGroup->setCheckable( TRUE );
	mainLayout->addWidget( p_buttonGroup );

	QVBoxLayout *vLayout1 = new QVBoxLayout( p_buttonGroup );
	QHBoxLayout *hLayout1 = new QHBoxLayout;
	QRadioButton *numRadio = new QRadioButton( i18n("Set number of chapers") ); // id = 0
	p_numChapter = new KIntSpinBox;
	p_numChapter->setRange( 2, 20 );
	connect( numRadio, SIGNAL( toggled( bool ) ), p_numChapter, SLOT( setEnabled( bool ) ) );
	hLayout1->addWidget( numRadio );
	hLayout1->addWidget( p_numChapter );
	vLayout1->addLayout( hLayout1 );

	QHBoxLayout *hLayout2 = new QHBoxLayout;
	QRadioButton *timeRadio = new QRadioButton( i18n("Set new chaper every") ); // id = 1
	p_timeChapter = new QTimeEdit;
	p_timeChapter->setEnabled( FALSE );
	p_timeChapter->setTimeRange( QTime( 0, 1 ), QTime( 1, 0 ) );
	connect( timeRadio, SIGNAL( toggled( bool ) ), p_timeChapter, SLOT( setEnabled( bool ) ) );
	hLayout2->addWidget( timeRadio );
	hLayout2->addWidget( p_timeChapter );
	vLayout1->addLayout( hLayout2 );

	QHBoxLayout *hLayout3 = new QHBoxLayout;
	QRadioButton *mediaRadio = new QRadioButton( i18n("Get chapters from Media Player") ); // id = 2
	KPushButton *getButton = new KPushButton( KIcon("list-add"), i18n("Get") );
	connect( getButton, SIGNAL( clicked() ), this, SLOT( getChapterFormPlayer() ) );
	connect( mediaRadio, SIGNAL( toggled( bool ) ), getButton, SLOT( setEnabled( bool ) ) );
	getButton->setEnabled( FALSE );
	hLayout3->addWidget( mediaRadio );
	hLayout3->addWidget( getButton );
	vLayout1->addLayout( hLayout3 );
	vLayout1->addStretch();

	QVBoxLayout *vLayout2 = new QVBoxLayout;
	mainLayout->addLayout( vLayout2 );
	QHBoxLayout *hLayout4 = new QHBoxLayout;
	KPushButton *iconView = new KPushButton( KIcon("view-list-icons"), i18n("Icon") );
	iconView->setCheckable( TRUE );
	p_viewMode->addButton( iconView, 0 );
	KPushButton *listView = new KPushButton( KIcon("view-list-text"), i18n("Text") );
	listView->setCheckable( TRUE );
	p_viewMode->addButton( listView, 1 );
	connect( p_viewMode, SIGNAL( buttonClicked( int ) ), this, SLOT( viewModeChanged( int ) ) );
	p_stopButton = new KPushButton( KIcon("process-stop"), i18n("Abort") );
	p_stopButton->setEnabled( FALSE );
	KPushButton *refreshButton = new KPushButton( KIcon("view-refresh"), i18n("Apply") );
	connect( refreshButton, SIGNAL( clicked() ), this, SLOT( createChapters() ) );
	KPushButton *clearButton = new KPushButton( KIcon("edit-clear-list"), i18n("Clear") );
	connect( clearButton, SIGNAL( clicked() ), this, SLOT( clearChapters() ) );
	hLayout4->addWidget( iconView );
	hLayout4->addWidget( listView );
	hLayout4->addWidget( p_stopButton );
	hLayout4->addWidget( refreshButton );
	hLayout4->addWidget( clearButton );
	vLayout2->addLayout( hLayout4 );

	p_list = new KListWidget;
	p_list->setMovement( QListView::Static );
	p_list->setResizeMode( QListView::Adjust );
	p_list->setContextMenuPolicy( Qt::CustomContextMenu );
	connect( p_list, SIGNAL( customContextMenuRequested( const QPoint& ) ), this, SLOT( showPopupMenu( const QPoint& ) ) );
	p_jobProgress = new QProgressBar;
	p_jobProgress->setMaximumHeight( 20 );
	vLayout2->addWidget( p_list );
	vLayout2->addWidget( p_jobProgress );

	connect( p_buttonGroup, SIGNAL( toggled( bool ) ), iconView, SLOT( setEnabled( bool ) ) );
	connect( p_buttonGroup, SIGNAL( toggled( bool ) ), listView, SLOT( setEnabled( bool ) ) );
	connect( p_buttonGroup, SIGNAL( toggled( bool ) ), p_list, SLOT( setEnabled( bool ) ) );
	connect( p_buttonGroup, SIGNAL( toggled( bool ) ), p_jobProgress, SLOT( setEnabled( bool ) ) );
	connect( p_buttonGroup, SIGNAL( toggled( bool ) ), refreshButton, SLOT( setEnabled( bool ) ) );
	connect( p_buttonGroup, SIGNAL( toggled( bool ) ), p_stopButton, SLOT( setEnabled( bool ) ) );
	connect( p_buttonGroup, SIGNAL( toggled( bool ) ), clearButton, SLOT( setEnabled( bool ) ) );

	p_buttonGroup->setSelected( 0 );
	listView->setChecked( TRUE );

	p_popup = new KMenu( p_list );
	KAction *del = new KAction( KIcon("edit-delete"), i18n("Delete"), this );
	connect( del, SIGNAL( triggered() ), this, SLOT( removeCurrentChapter() ) );
	p_popup->addAction( del );
}

KDCChapterEditor::~KDCChapterEditor()
{
	if ( p_job )
		p_job->deleteLater();
	delete p_popup;
}

void KDCChapterEditor::setCurrentMovie( KDCMovie *movie )
{
	if ( !movie )
		return;

	if ( p_movie == movie )
		return;

	clean();
	p_movie = movie;

	if ( p_movie->chapterGroup()->viewMode() == KDC::List )
		p_viewMode->button( 1 )->setChecked( TRUE );
	else
		p_viewMode->button( 0 )->setChecked( TRUE );
	refresh();
}

void KDCChapterEditor::setCurrentMovie( KDCMovieGroup *group )
{
	qDebug( "KDCChapterEditor::setCurrentMovieGroup( %d )", KDCLib::isNull( group ) );
	if ( !group )
		return;

	qDebug( "KDCChapterEditor::setCurrentMovieGroup()::count = %d", group->count() );

	if ( p_group == group )
		return;

	clean();
	p_group = group;

	if ( p_group->chapterGroup()->viewMode() == KDC::List )
		p_viewMode->button( 1 )->setChecked( TRUE );
	else
		p_viewMode->button( 0 )->setChecked( TRUE );
	refresh();
	qDebug( "KDCChapterEditor::setCurrentMovieGroup()::end" );
}

void KDCChapterEditor::clean()
{
	if ( p_job )
		p_job->stop();
	if ( p_movie )
		KDCFileFactory::removeDirectory( KDCFileFactory::movieDir( p_movie ) );
	if ( p_group )
		KDCFileFactory::removeDirectory( KDCFileFactory::movieDir( p_group ) );
	p_movie= NULL;
	p_group = NULL;

	p_list->clear();
	p_jobProgress->setValue( 0 );
}

void KDCChapterEditor::createPreview()
{
	qDebug( "KDCChapterEditor::createPreview()::begin" );
	if ( !p_movie && !p_group )
		return;

	if ( p_job )
		delete p_job;

	if( currentChapterGroup()->count() == 0 )
		return;

	if ( p_movie )
		p_job = new KDCChaptersPreviewCompositeJob( p_movie );
	else
		p_job = new KDCChaptersPreviewCompositeJob( p_group );
	p_jobProgress->setMaximum( p_job->totalSize() );
	connect( p_job, SIGNAL( percent( int ) ), p_jobProgress, SLOT( setValue( int ) ) );
	connect( p_job, SIGNAL( finished( KDCJob * ) ), this, SLOT( previewFinished( KDCJob * ) ) );
	connect( p_job, SIGNAL( previewAvalible( const QString& , int ) ), this, SLOT( on_prevAvalible( const QString& , int ) ) );
	connect( p_stopButton, SIGNAL( clicked() ), p_job, SLOT( stop() ) );
	p_job->start();
	p_stopButton->setEnabled( TRUE );
	qDebug( "KDCChapterEditor::createPreview::end" );
}

void KDCChapterEditor::previewFinished( KDCJob *job )
{
	if ( job->hasErrors() )
		KMessageBox::error( this, i18n("<b>There was an error during prosess:</b>%1").arg( job->error() ) );
	job->deleteLater();
	p_stopButton->setEnabled( FALSE );
}

void KDCChapterEditor::on_prevAvalible( const QString& fileName, int second )
{
	QTime time;
	p_list->addItem( new QListWidgetItem( QIcon( QPixmap( fileName ) ), time.addSecs( second ).toString("hh:mm:ss") ) );
}

void KDCChapterEditor::readChapters()
{
	qDebug( "KDCChapterEditor::readChapters()::begin" );
	if ( !p_movie && !p_group )
		return;

	p_list->clear();
	KDCChapterGroup *group = currentChapterGroup();

	QTime time;
	QString str;
	for ( int i=0; i<group->count(); ++i )
	{
		str = QString( i18n("chapter %1 - %2") ).arg( i + 1 );
		str = str.arg( time.addSecs( group->chapter( i ) ).toString("hh:mm:ss") );
		p_list->addItem( new QListWidgetItem( str ) );
	}
	qDebug( "KDCChapterEditor::readChapters()::end" );
}

void KDCChapterEditor::createChapters()
{
	qDebug( "KDCChapterEditor::createChapters()::begin" );
	if ( !p_movie && !p_group )
		return;

	KDCChapterGroup *group = currentChapterGroup();

	if ( p_buttonGroup->selected() == 0 )
		group->createFromSize( p_numChapter->value() );
	else
		if ( p_buttonGroup->selected() == 1 )
		{
			QTime time = p_timeChapter->time();
			group->createFromLength( ( time.hour()*60.0 + time.minute() )*60.0 + time.second() );
		}
	refresh();
	qDebug( "KDCChapterEditor::createChapters()::end" );
}

void KDCChapterEditor::viewModeChanged( int id )
{
	if ( p_id != id )
	{
		p_id = id;
		refresh();
	}
}

void KDCChapterEditor::refresh()
{
	qDebug( "KDCChapterEditor::refresh()::begin" );
	if ( !p_buttonGroup->isChecked() )
		return;

	if ( !p_movie && !p_group )
		return;

	if ( p_job )
		delete p_job;

	if ( p_viewMode->checkedId() == 0 )
	{
		p_list->setViewMode( QListView::IconMode );
		p_list->setIconSize( QSize( 112, 63 ) );
		p_list->setSpacing( 12 );
		p_list->clear();
		createPreview();
		currentChapterGroup()->setViewMode( KDC::Icon );
	}
	else
	{
		p_list->setViewMode( QListView::ListMode );
		p_list->setIconSize( QSize( 22, 22 ) );
		p_list->setSpacing( 0 );
		p_list->clear();
		readChapters();
		currentChapterGroup()->setViewMode( KDC::List );
	}
	qDebug( "KDCChapterEditor::refresh()::end" );
}

KDCChapterGroup* KDCChapterEditor::currentChapterGroup()
{
	if ( p_movie )
		return p_movie->chapterGroup();
	else
		if ( p_group )
			return p_group->chapterGroup();
	else
		return NULL;
}

void KDCChapterEditor::getChapterFormPlayer()
{
	KDCChapterGroup *group = currentChapterGroup();
	if ( group )
	{
		group->add( p_mediaPlayer->currentPos() );
		refresh();
	}
}

void KDCChapterEditor::clearChapters()
{
	KDCChapterGroup *group = currentChapterGroup();
	if ( group )
	{
		group->clear();
		p_list->clear();
	}
}

void KDCChapterEditor::removeCurrentChapter()
{
	currentChapterGroup()->remove( p_list->currentRow() );
	delete p_list->takeItem( p_list->currentRow() );
	if ( p_viewMode->checkedId() == 1 )
		refresh();
}

void KDCChapterEditor::showPopupMenu( const QPoint& pos )
{
	if ( p_list->count() > 0 && p_list->currentRow() != -1 )
		p_popup->exec( p_list->mapToGlobal( pos ) );
}
