/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcprogressbar.h"

#include "math.h"

KDCProgressBar::KDCProgressBar( QWidget *parent )
	: QProgressBar( parent )
{
	p_totalSize = 0;
	p_percent = 0;
	p_value = 0;
}

KDCProgressBar::~KDCProgressBar()
{

}

void KDCProgressBar::setTotalSize( int size )
{
	if ( size > 0 )
		createRanges( size );
	it = p_ranges.constBegin();
	p_totalSize = size;
	p_percent = 0;
	p_value = 0;
}

int KDCProgressBar::percent() const
{
	if ( p_percent > 100 )
		return 100;
	else
		return p_percent;
}

int KDCProgressBar::currentValue() const
{
	return p_value;
}

void KDCProgressBar::refresh()
{
	setProgress( p_value );
}

int KDCProgressBar::totalSize() const
{
	return p_totalSize;
}

void KDCProgressBar::setNewValue( int val )
{
	p_value = val;
}

void KDCProgressBar::setProgress( int val )
{
	p_value = val;
	int tmp = p_percent;
	while ( it != p_ranges.constEnd() )
	{
		if ( *it >= val )
		{
			setValue( tmp );
			break;
		}
		++it;
		++tmp;
	}
	emit percentChanged( p_percent, tmp );
	p_percent = tmp;
}

void KDCProgressBar::setPercent( int val )
{
	emit percentChanged( p_percent, val );
	p_percent = val;
	setValue( val );
}

void KDCProgressBar::createRanges( const int max )
{
	double tmp = (double)max / 100.0;
	p_ranges.clear();
	for ( int i=1; i<=100; ++i )
		p_ranges.append( floor( tmp*i ) );
}
