/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcvideoqualitydelegate.h"

#include "kdcvideoqualitycombobox.h"

#include <QModelIndex>

KDCVideoQualityDelegate::KDCVideoQualityDelegate( KDCVideoQualityDelegate::Mode mode, QObject *parent )
	: QItemDelegate( parent )
{
	p_mode = mode;
}

void KDCVideoQualityDelegate::setCurrentData( const KDCMovie *movie )
{
	p_movie = movie;
}

QWidget* KDCVideoQualityDelegate::createEditor( QWidget *parent, const QStyleOptionViewItem& option, const QModelIndex& index ) const
{
	if ( p_mode == KDCVideoQualityDelegate::MacroBlock )
	{
		KDCVideoQualityComboBox *editor = new KDCVideoQualityComboBox(	p_movie, KDCVideoQualityComboBox::MacroBlock,
																																		parent );
		connect(	editor, SIGNAL( currentMacroBlockModeChanged( KDC::MacroBlockMode ) ), this,
							SLOT( on_currentMacroBlockModeChanged( KDC::MacroBlockMode ) ) );
		return editor;
	}
	else
	{
		KDCVideoQualityComboBox *editor = new KDCVideoQualityComboBox(	p_movie, KDCVideoQualityComboBox::Deinterlace,
																																		parent );
		connect(	editor, SIGNAL( currentDeinterlaceModeChanged( KDC::DeinterlaceMode ) ), this,
							SLOT( on_currentDeinterlaceModeChanged( KDC::DeinterlaceMode ) ) );
		return editor;
	}
}

void KDCVideoQualityDelegate::setEditorData( QWidget *editor, const QModelIndex& index ) const
{
	//QString codec = index.model()->data( index, Qt::DisplayRole ).toString();
	//KDCEncodingComboBox *tmp = static_cast<KDCEncodingComboBox*>( editor );
	//tmp->setCurrentCodec( p_codec );
}

void KDCVideoQualityDelegate::setModelData( QWidget *editor, QAbstractItemModel *model, const QModelIndex& index) const
{
	KDCVideoQualityComboBox *tmp = static_cast<KDCVideoQualityComboBox*>( editor );
	model->setData( index, tmp->currentText() );
}

void KDCVideoQualityDelegate::updateEditorGeometry( QWidget *editor, const QStyleOptionViewItem& option, const QModelIndex& index) const
{
	editor->setGeometry( option.rect );
}

void KDCVideoQualityDelegate::on_currentMacroBlockModeChanged( KDC::MacroBlockMode mode )
{
	emit macroBlockModeChanged( mode );
}

void KDCVideoQualityDelegate::on_currentDeinterlaceModeChanged( KDC::DeinterlaceMode mode )
{
	emit deinterlaceModeChanged( mode );
}
